#include "file.h"

/* Gestion des fichiers TGA */
static int freadint(FILE *fp)
{
    unsigned char buffer[4];
    fread(buffer, 4, 1, fp);
    return ((int)buffer[3] << 24) | ((int)buffer[2] << 16)
             | ((int)buffer[1] << 8) | ((int)buffer[0]);
}
                                                                                
static int freadshort(FILE *fp)
{
    unsigned char buffer[2];
    fread(buffer, 2, 1, fp);
    return ((int)buffer[1] << 8) | ((int)buffer[0]);
}
                                                                                
static int freadchar(FILE *fp)
{
    unsigned char buffer;
    fread(&buffer, 1, 1, fp);
    return (int)buffer;
}
char * read_tga(char *name,int *w,int *h,int *depth)
{
    char *pixels;
    FILE *fp;
    static int tmp;
    int bpp;
    int rmask,gmask,bmask,amask,i;
    fp = fopen(name, "rb");
    if(!fp)
    {
	printf("Erreur chargement TGA...\n");
        return NULL;
    }
                                                                                
    tmp = freadshort(fp);
    fseek(fp, 0, SEEK_SET);
                                                                                
    if(tmp == 0x4d42)
    {
        /* Read a BMP file */
    }
    else
    {
        /* Read a TGA file */
                                                                                
        tmp = freadchar(fp); /* ID length */
        if(freadchar(fp) != 0x00) /* Colormap type */
        {
            fclose(fp);
            return NULL;
        }
        if(freadchar(fp) != 0x02) /* Image type */
        {
            fclose(fp);
            return NULL;
        }
                                                                                
        /* Skip Colormap specification */
        fseek(fp, 5, SEEK_CUR);
                                                                                
        /* Read Image specification */
        *w = freadshort(fp); /* X origin */
        *h = freadshort(fp); /* Y origin */
        *w = freadshort(fp);
        *h = freadshort(fp);
        bpp = freadchar(fp);
        freadchar(fp); /* image descriptor */
                                                                                
        /* Skip image ID */
        fseek(fp, tmp, SEEK_CUR);
                                                                                
        /* Fill in more stuff */
        (*depth) = (bpp + 7) / 8;
                                                                                
        switch((*depth))
        {
        case 3:
            rmask = 0xff0000;
            gmask = 0x00ff00;
            bmask = 0x0000ff;
            amask = 0x000000;
            break;
        case 2: /* XXX: those are the 16 bits values */
            rmask = 0x7c00;
            gmask = 0x03e0;
            bmask = 0x001f;
            amask = 0x0000;
            break;
        case 1:
        default:
            bpp = 8;
            rmask = gmask = bmask = amask = 0;
            break;
        }
                                                                                
        /* Allocate the pixel buffer */
        pixels = malloc((*w) * (*h) * (*depth));
        if(!pixels)
        {
            fclose(fp);
            return NULL;
        }
                                                                                
        memset(pixels, 0, (*w) * (*h) * (*depth));
                                                                                
        for(i = *h; i--; /* */)
            fread(pixels + (*w) * (*depth) * i, (*w) * (*depth), 1, fp);
    }
                                                                                
    fclose(fp);
    return pixels;
}
IplImage *ChartoIpl(char *pixels,int w,int h,int depth)
{
	IplImage *tmp;
	int step;
	tmp = cvCreateImage (cvSize(w,h), 8, 3);
	step = (w*3 + 3) & -4;
	step = (w*depth);
	cvInitImageHeader(tmp, cvSize(w,h), IPL_DEPTH_8U, 3 /* chan */,
			IPL_ORIGIN_TL, 4 /* align */);
	cvSetImageData(tmp, pixels, step);
	return tmp;
}
IplImage *ImageLoad(char *Name)
{
	IplImage *tmp;
	char *pixels;
	int w,h,depth;
	if(strstr( Name, "tga")!= NULL)
	{
		// Attention : mauvaises valeurs de 'w' et 'h'
		pixels = read_tga(Name,&w,&h,&depth);
		tmp = ChartoIpl(pixels,w,h,depth);
	}
	else
		tmp = cvLoadImage(Name, -1);
	return tmp;
}
